def __api_endpoints():
    """
    Returns a tuple of three strings containing API endpoints for SearchCode.com.
    code_search_endpoint: API endpoint for searching for code snippets on SearchCode.
        Takes in the following parameters in order:
            - Search query
            - Page number
            - Number of results per page
            - Source parameter (optional)
            - Language parameter (optional)

    code_result_endpoint: API endpoint for retrieving details about a specific code search result.
    Takes in the ID of the code search result as a parameter.

    related_results_endpoint: API endpoint for retrieving related code search results.
    Takes in the ID of the code search result as a parameter.
    """
    api = "https://searchcode.com/api/"
    code_search_endpoint = api + "codesearch_I/?q={}&p={}&per_page={}{}{}"
    code_result_endpoint = api + "result/{}"
    related_results_endpoint = api + "related_results/{}"

    return code_search_endpoint, code_result_endpoint, related_results_endpoint


def __code_sources():
    """
    Returns a list of tuples containing code sources and their corresponding source parameters.

    Returns:
    --------
    A list of tuples in the format: (SOURCE_NAME, SOURCE_PARAMETER)

    Note:
    ----
    The SOURCE_NAME indicates the name of the code source, while the SOURCE_PARAMETER is a string that can be appended to
    the end of a URL to specify the source of the code.
    """
    return [("Google Code", "&src=1"),
            ("GitHub", "&src=2"),
            ("BitBucket", "&src=3"),
            ("Sourceforge", "&src=4"),
            ("CodePlex", "&src=5"),
            ("Minix3", "&src=6"),
            ("Fedora Project", "&src=7"),
            ("Seek Quarry", "&src=8"),
            ("Tizen", "&src=9"),
            ("Gitorious", "&src=10"),
            ("Google Android", "&src=12"),
            ("GitLab", "&src=13"),
            ("Codeberg", "&src=14"),
            ("Repo.or.cz", "&src=15"),
            ("Sr.ht", "&src=16")]


def __code_languages():
    """
    Returns a list of tuples containing code languages and their corresponding language parameters.

    Returns:
    --------
    A list of tuples in the format: (LANGUAGE_NAME, LANGUAGE_PARAMETER)

    Note:
    -----
    The LANGUAGE_NAME indicates the name of the programming language, while the LANGUAGE_PARAMETER is a string that can be
    appended to the end of a URL to specify the language of the code.
    """
    return [("XAML", "&lan=1"),
            ("ASP.NET", "&lan=2"),
            ("HTML", "&lan=3"),
            ("Unknown", "&lan=4"),
            ("MSBuild scripts", "&lan=5"),
            ("C#", "&lan=6"),
            ("XSD", "&lan=7"),
            ("XML", "&lan=8"),
            ("CMake", "&lan=14"),
            ("C++ Header", "&lan=15"),
            ("C++", "&lan=16"),
            ("Makefile", "&lan=17"),
            ("CSS", "&lan=18"),
            ("Python", "&lan=19"),
            ("MATLAB", "&lan=20"),
            ("Objective C", "&lan=21"),
            ("JavaScript", "&lan=22"),
            ("Java", "&lan=23"),
            ("PHP", "&lan=24"),
            ("Erlang", "&lan=25"),
            ("FORTRAN Legacy", "&lan=26"),
            ("FORTRAN Modern", "&lan=27"),
            ("C", "&lan=28"),
            ("Lisp", "&lan=29"),
            ("Visual Basic", "&lan=30"),
            ("Shell", "&lan=31"),
            ("Ruby", "&lan=32"),
            ("Vim Script", "&lan=33"),
            ("Assembly", "&lan=34"),
            ("Objective C++", "&lan=35"),
            ("Document Type Definition", "&lan=36"),
            ("SQL", "&lan=37"),
            ("YAML", "&lan=38"),
            ("Ruby HTML", "&lan=39"),
            ("Haskell", "&lan=40"),
            ("Bash", "&lan=41"),
            ("ActionScript", "&lan=42"),
            ("MXML", "&lan=43"),
            ("ASP", "&lan=44"),
            ("D", "&lan=45"),
            ("Pascal", "&lan=46"),
            ("Scala", "&lan=47"),
            ("Batch", "&lan=48"),
            ("Groovy", "&lan=49"),
            ("Extensible Stylesheet Language Transformations", "&lan=50"),
            ("Perl", "&lan=51"),
            ("Teamcenter def", "&lan=52"),
            ("IDL", "&lan=53"),
            ("Lua", "&lan=54"),
            ("Go", "&lan=55"),
            ("yacc", "&lan=56"),
            ("Cython", "&lan=57"),
            ("LEX", "&lan=59"),
            ("Ada", "&lan=61"),
            ("sed", "&lan=62"),
            ("m4", "&lan=63"),
            ("OCaml", "&lan=64"),
            ("Smarty Template", "&lan=65"),
            ("ColdFusion", "&lan=66"),
            ("NAnt scripts", "&lan=67"),
            ("Expect", "&lan=68"),
            ("C Shell", "&lan=69"),
            ("VHDL", "&lan=70"),
            ("TCL", "&lan=71"),
            ("JavaServer Pages", "&lan=72"),
            ("SKILL", "&lan=73"),
            ("AWK", "&lan=74"),
            ("MUMPS", "&lan=75"),
            ("SQL Data", "&lan=76"),
            ("Korn Shell", "&lan=78"),
            ("Patran Command Language", "&lan=83"),
            ("DAL", "&lan=84"),
            ("Fortran 95", "&lan=85"),
            ("Octave", "&lan=86"),
            ("Oracle Forms", "&lan=87"),
            ("Dart", "&lan=88"),
            ("COBOL", "&lan=89"),
            ("Modula3", "&lan=90"),
            ("Rexx", "&lan=91"),
            ("Oracle Reports", "&lan=92"),
            ("Softbridge Basic", "&lan=93"),
            ("bc", "&lan=94"),
            ("Teamcenter met", "&lan=95"),
            ("Kermit", "&lan=98"),
            ("Teamcenter mth", "&lan=99"),
            ("AMPLE", "&lan=100"),
            ("CCS", "&lan=101"),
            ("JCL", "&lan=102"),
            ("ABAP", "&lan=103"),
            ("Clojure", "&lan=104"),
            ("OpenCL", "&lan=105"),
            ("CoffeeScript", "&lan=106"),
            ("QML", "&lan=107"),
            ("AutoHotkey", "&lan=108"),
            ("ClojureScript", "&lan=109"),
            ("ColdFusion CFScript", "&lan=110"),
            ("gitignore", "&lan=111"),
            ("Config", "&lan=113"),
            ("Patch", "&lan=114"),
            ("Markdown", "&lan=118"),
            ("JSON", "&lan=122"),
            ("Portable Object", "&lan=126"),
            ("Certificate", "&lan=128"),
            ("Hg Ignore", "&lan=129"),
            ("MSBuild", "&lan=130"),
            ("Windows Module Definition", "&lan=131"),
            ("HLSL", "&lan=132"),
            ("Sass", "&lan=133"),
            ("LESS", "&lan=135"),
            ("CUDA", "&lan=136"),
            ("Swift", "&lan=137"),
            ("Maven", "&lan=139"),
            ("Visualforce Component", "&lan=140"),
            ("Verilog-SystemVerilog", "&lan=141"),
            ("JavaServer Faces", "&lan=142"),
            ("Racket", "&lan=143"),
            ("R", "&lan=144"),
            ("Kotlin", "&lan=145"),
            ("Powershell", "&lan=146"),
            ("Rust", "&lan=147"),
            ("Velocity Template Language", "&lan=148"),
            ("Razor", "&lan=149"),
            ("F#", "&lan=150"),
            ("TypeScript", "&lan=151"),
            ("NAnt script", "&lan=152"),
            ("Ant", "&lan=153"),
            ("Arduino Sketch", "&lan=154"),
            ("Haml", "&lan=155"),
            ("Grails", "&lan=156"),
            ("Puppet", "&lan=158"),
            ("Vala", "&lan=159"),
            ("Windows Resource File", "&lan=160"),
            ("Unity-Prefab", "&lan=161"),
            ("Handlebars", "&lan=162"),
            ("Robot Framework", "&lan=163"),
            ("Pig Latin", "&lan=164"),
            ("WiX source", "&lan=165"),
            ("WiX include", "&lan=166"),
            ("Mustache", "&lan=167"),
            ("Windows Message File", "&lan=168"),
            ("XQuery", "&lan=169"),
            ("ECPP", "&lan=172"),
            ("Visualforce Page", "&lan=173"),
            ("WiX string localization", "&lan=174"),
            ("Apex Trigger", "&lan=175"),
            ("Vala Header", "&lan=176"),
            ("xBase Header", "&lan=177"),
            ("xBase", "&lan=178"),
            ("InstallShield", "&lan=179"),
            ("Harbour", "&lan=180"),
            ("Forth", "&lan=181"),
            ("PL/I", "&lan=182"),
            ("CSON", "&lan=183"),
            ("TeX", "&lan=184"),
            ("Brainfuck", "&lan=185"),
            ("Elixir", "&lan=186"),
            ("zsh", "&lan=187"),
            ("Julia", "&lan=188"),
            ("dtrace", "&lan=189"),
            ("Mathematica", "&lan=190"),
            ("Standard ML", "&lan=191"),
            ("SAS", "&lan=192"),
            ("Haxe", "&lan=193"),
            ("Nim", "&lan=194"),
            ("TTCN", "&lan=195"),
            ("Mercury", "&lan=196"),
            ("Clean", "&lan=197"),
            ("Prolog", "&lan=198"),
            ("PureScript", "&lan=199"),
            ("ERB", "&lan=200"),
            ("Stylus", "&lan=201"),
            ("XHTML", "&lan=202"),
            ("Qt Project", "&lan=203"),
            ("diff", "&lan=204"),
            ("INI", "&lan=205"),
            ("JSX", "&lan=206"),
            ("Qt Linguist", "&lan=207"),
            ("Qt", "&lan=208"),
            ("Jam", "&lan=209"),
            ("Protocol Buffers", "&lan=210"),
            ("liquid", "&lan=211"),
            ("Pug", "&lan=212"),
            ("Freemarker Template", "&lan=213"),
            ("XMI", "&lan=214"),
            ("ClojureC", "&lan=215"),
            ("Titanium Style Sheet", "&lan=216"),
            ("Coq", "&lan=217"),
            ("Crystal", "&lan=218"),
            ("AspectJ", "&lan=220"),
            ("EEx", "&lan=221"),
            ("Elm", "&lan=222"),
            ("Twig", "&lan=223"),
            ("Slim", "&lan=224"),
            ("Logtalk", "&lan=225"),
            ("GDScript", "&lan=226"),
            ("PowerBuilder", "&lan=227"),
            ("Blade", "&lan=228"),
            ("builder", "&lan=229"),
            ("Visual Fox Pro", "&lan=230"),
            ("SQL Stored Procedure", "&lan=231"),
            ("License", "&lan=232"),
            ("Agda", "&lan=233"),
            ("Alchemist", "&lan=234"),
            ("Alex", "&lan=235"),
            ("Alloy", "&lan=236"),
            ("Android Interface Definition Language", "&lan=237"),
            ("Arvo", "&lan=238"),
            ("AsciiDoc", "&lan=239"),
            ("ATS", "&lan=240"),
            ("Autoconf", "&lan=241"),
            ("Basic", "&lan=242"),
            ("Bazel", "&lan=243"),
            ("Bitbake", "&lan=244"),
            ("Bitbucket Pipeline", "&lan=245"),
            ("Boo", "&lan=246"),
            ("Bosque", "&lan=247"),
            ("BuildStream", "&lan=248"),
            ("C Header", "&lan=249"),
            ("Cabal", "&lan=250"),
            ("Cargo Lock", "&lan=251"),
            ("Cassius", "&lan=252"),
            ("Ceylon", "&lan=253"),
            ("Closure Template", "&lan=254"),
            ("Cogent", "&lan=255"),
            ("Creole", "&lan=256"),
            ("CSV", "&lan=257"),
            ("Device Tree", "&lan=258"),
            ("Dhall", "&lan=259"),
            ("Docker ignore", "&lan=260"),
            ("Dockerfile", "&lan=261"),
            ("Emacs Dev Env", "&lan=262"),
            ("Emacs Lisp", "&lan=263"),
            ("F*", "&lan=265"),
            ("FIDL", "&lan=266"),
            ("Fish", "&lan=267"),
            ("Flow9", "&lan=268"),
            ("Fragment Shader File", "&lan=269"),
            ("Futhark", "&lan=270"),
            ("Game Maker Language", "&lan=271"),
            ("Game Maker Project", "&lan=272"),
            ("Gemfile", "&lan=273"),
            ("Gherkin Specification", "&lan=274"),
            ("GLSL", "&lan=275"),
            ("GN", "&lan=276"),
            ("Go Template", "&lan=277"),
            ("Gradle", "&lan=278"),
            ("Hamlet", "&lan=279"),
            ("Happy", "&lan=280"),
            ("HEX", "&lan=281"),
            ("Idris", "&lan=282"),
            ("ignore", "&lan=283"),
            ("Intel HEX", "&lan=284"),
            ("Isabelle", "&lan=285"),
            ("Jade", "&lan=286"),
            ("JAI", "&lan=287"),
            ("Janet", "&lan=288"),
            ("Jenkins Buildfile", "&lan=289"),
            ("Jinja", "&lan=290"),
            ("JSONL", "&lan=291"),
            ("Julius", "&lan=292"),
            ("Jupyter", "&lan=293"),
            ("Just", "&lan=294"),
            ("LaTeX", "&lan=295"),
            ("LD Script", "&lan=296"),
            ("Lean", "&lan=297"),
            ("LOLCODE", "&lan=298"),
            ("Lucius", "&lan=299"),
            ("Luna", "&lan=300"),
            ("Macromedia eXtensible Markup Language", "&lan=301"),
            ("Madlang", "&lan=302"),
            ("Mako", "&lan=303"),
            ("Meson", "&lan=304"),
            ("Module-Definition", "&lan=305"),
            ("Monkey C", "&lan=306"),
            ("MQL Header", "&lan=307"),
            ("MQL4", "&lan=308"),
            ("MQL5", "&lan=309"),
            ("Nix", "&lan=310"),
            ("nuspec", "&lan=311"),
            ("Opalang", "&lan=312"),
            ("Org", "&lan=313"),
            ("Oz", "&lan=314"),
            ("PKGBUILD", "&lan=315"),
            ("PL/SQL", "&lan=316"),
            ("Plain Text", "&lan=317"),
            ("Polly", "&lan=318"),
            ("Pony", "&lan=319"),
            ("Processing", "&lan=320"),
            ("Properties File", "&lan=321"),
            ("PSL Assertion", "&lan=322"),
            ("Q#", "&lan=323"),
            ("QCL", "&lan=324"),
            ("Rakefile", "&lan=325"),
            ("Report Definition Language", "&lan=326"),
            ("ReStructuredText", "&lan=327"),
            ("Scheme", "&lan=328"),
            ("Scons", "&lan=329"),
            ("SPDX", "&lan=330"),
            ("Specman e", "&lan=331"),
            ("Spice Netlist", "&lan=332"),
            ("SRecode Template", "&lan=333"),
            ("Stata", "&lan=334"),
            ("SVG", "&lan=335"),
            ("Swig", "&lan=336"),
            ("Systemd", "&lan=337"),
            ("SystemVerilog", "&lan=338"),
            ("TaskPaper", "&lan=339"),
            ("Terraform", "&lan=341"),
            ("Thrift", "&lan=342"),
            ("TOML", "&lan=343"),
            ("Twig Template", "&lan=344"),
            ("TypeScript Typings", "&lan=345"),
            ("Unreal Script", "&lan=346"),
            ("Ur/Web", "&lan=347"),
            ("Ur/Web Project", "&lan=348"),
            ("V", "&lan=349"),
            ("Varnish Configuration", "&lan=350"),
            ("Verilog", "&lan=351"),
            ("Verilog Args File", "&lan=352"),
            ("Vertex Shader File", "&lan=353"),
            ("Visual Basic for Applications", "&lan=354"),
            ("Vue", "&lan=355"),
            ("Web Services Description Language", "&lan=356"),
            ("Wolfram", "&lan=357"),
            ("Wren", "&lan=358"),
            ("Xcode Config", "&lan=359"),
            ("XML Schema", "&lan=360"),
            ("Xtend", "&lan=361"),
            ("Yarn", "&lan=362"),
            ("Zig", "&lan=363"),
            ("bait", "&lan=364"),
            ("CloudFormation (JSON)", "&lan=365"),
            ("CloudFormation (YAML)", "&lan=366"),
            ("CodeQL", "&lan=367"),
            ("DM", "&lan=368"),
            ("Fennel", "&lan=369"),
            ("FSL", "&lan=370"),
            ("FXML", "&lan=371"),
            ("hoon", "&lan=372"),
            ("Nial", "&lan=373"),
            ("ReasonML", "&lan=374"),
            ("Sieve", "&lan=375"),
            ("Solidity", "&lan=376"),
            ("Teal", "&lan=377"),
            ("TL", "&lan=378")]
